import sys

from PySide6.QtCore import (QFile, QCoreApplication, QDate, QDateTime, QLocale, QMetaObject, QObject, QPoint, QRect,
                            QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor, QFont, QFontDatabase, QGradient, QIcon, QImage,
                           QKeySequence, QLinearGradient, QPainter, QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QDial, QDialog, QDoubleSpinBox, QFrame, QLabel, QLineEdit,
                               QMainWindow, QMenuBar, QPushButton, QRadioButton, QSizePolicy, QSlider, QSpinBox,
                               QStatusBar, QTextEdit, QWidget)

from ui_ct import Ui_MainWindow

steps_int: int = 0
irange_arr = ["x1", "x.5", "x.2", "x.1", "x.05", "x.02"]
irange_arr_val = [1, .5, .2, .1, .05, .02]
irange_int: int = 0
output_arr = ["50nA/uV", "100nA/uV", "200nA/uV", "500nA/uV", "1uA/mV", "2uA/mV", "5uA/mV", "10uA/mV", "20uA/mV",
              "50uA/mV", "100uA/mV", "200uA/mV", "500uA/mV", "1mA/V", "2mA/V", "5mA/V"]
output_int: int = 0
type_arr = ["BJT", "FET"]
pol_arr = ["PNP", "NPN"]
vout_arr = ["35V / 2A", "70V / 1A", "200V / 0.1A"]


class Window(QMainWindow, Ui_MainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)

        self.doublespinbox_vout.setMaximum(35)
        self.doublespinbox_vout.setSingleStep(.1)
        self.doublespinbox_vout.setDecimals(1)
        self.dial_pot_vrange.setMaximum(350)
        self.dial_pot_vrange.setSingleStep(1)

        def step_func():
            global steps_int
            steps_int = int((self.dial_rsw_steps.value() - 3) / 2)
            self.dial_rsw_steps.setValue(steps_int * 2 + 3)
            if steps_int < 0:
                steps_int = 0
                self.dial_rsw_steps.setValue(3)
            if steps_int > 7:
                steps_int = 7
                self.dial_rsw_steps.setValue(17)
            # print("Dial Value = %i" % (steps_int))
            self.spinbox_steps.setValue(steps_int)
            if steps_int == 7:
                self.radiobtn_steps_7.click()
            elif steps_int == 6:
                self.radiobtn_steps_6.click()
            elif steps_int == 5:
                self.radiobtn_steps_5.click()
            elif steps_int == 4:
                self.radiobtn_steps_4.click()
            elif steps_int == 3:
                self.radiobtn_steps_3.click()
            elif steps_int == 2:
                self.radiobtn_steps_2.click()
            elif steps_int == 1:
                self.radiobtn_steps_1.click()
            elif steps_int == 0:
                self.radiobtn_steps_0.click()

        def stepsel_func():
            self.dial_rsw_steps.setValue(self.spinbox_steps.value() * 2 + 3)

        def step0_func():
            self.dial_rsw_steps.setValue(0 * 2 + 3)

        def step1_func():
            self.dial_rsw_steps.setValue(1 * 2 + 3)

        def step2_func():
            self.dial_rsw_steps.setValue(2 * 2 + 3)

        def step3_func():
            self.dial_rsw_steps.setValue(3 * 2 + 3)

        def step4_func():
            self.dial_rsw_steps.setValue(4 * 2 + 3)

        def step5_func():
            self.dial_rsw_steps.setValue(5 * 2 + 3)

        def step6_func():
            self.dial_rsw_steps.setValue(6 * 2 + 3)

        def step7_func():
            self.dial_rsw_steps.setValue(7 * 2 + 3)

        def irange_func():
            global irange_int
            irange_int = int((self.dial_rsw_irange.value() - 5) / 2)
            self.dial_rsw_irange.setValue(irange_int * 2 + 5)
            self.combobox_irange.setCurrentIndex(irange_int)
            global irange_arr_val
            if self.slider_vrange.value() == 0:
                iout = 2000
            elif self.slider_vrange.value() == 1:
                iout = 1000
            else:
                iout = 100
            self.dial_pot_irange.setValue(int(self.spinbox_iout.value() / iout / irange_arr_val[irange_int] * 100))
            if irange_int < 0:
                irange_int = 0
                self.dial_rsw_irange.setValue(5)
            if irange_int > 5:
                irange_int = 5
                self.dial_rsw_irange.setValue(15)
            if irange_int == 5:
                self.radiobtn_irange_x002.click()
            elif irange_int == 4:
                self.radiobtn_irange_x005.click()
            elif irange_int == 3:
                self.radiobtn_irange_x01.click()
            elif irange_int == 2:
                self.radiobtn_irange_x02.click()
            elif irange_int == 1:
                self.radiobtn_irange_x05.click()
            elif irange_int == 0:
                self.radiobtn_irange_x1.click()

        def irangesel_func():
            self.dial_rsw_irange.setValue(self.combobox_irange.currentIndex() * 2 + 5)

        def irange0_func():
            self.dial_rsw_irange.setValue(0 * 2 + 5)

        def irange1_func():
            self.dial_rsw_irange.setValue(1 * 2 + 5)

        def irange2_func():
            self.dial_rsw_irange.setValue(2 * 2 + 5)

        def irange3_func():
            self.dial_rsw_irange.setValue(3 * 2 + 5)

        def irange4_func():
            self.dial_rsw_irange.setValue(4 * 2 + 5)

        def irange5_func():
            self.dial_rsw_irange.setValue(5 * 2 + 5)

        def output_func():
            global output_int
            output_int = self.dial_rsw_out.value()
            if output_int > 15:
                output_int = 15
                self.dial_rsw_out.setValue(15)
            if output_int == 15:
                self.radiobtn_out_5mAV.click()
                self.slider_yamp.setValue(0)
            elif output_int == 14:
                self.radiobtn_out_2mAV.click()
                self.slider_yamp.setValue(0)
            elif output_int == 13:
                self.radiobtn_out_1mAV.click()
                self.slider_yamp.setValue(0)
            elif output_int == 12:
                self.radiobtn_out_500uAmV.click()
                self.slider_yamp.setValue(0)
            elif output_int == 11:
                self.radiobtn_out_200uAmV.click()
                self.slider_yamp.setValue(0)
            elif output_int == 10:
                self.radiobtn_out_100uAmV.click()
                self.slider_yamp.setValue(1)
            elif output_int == 9:
                self.radiobtn_out_50uAmV.click()
                self.slider_yamp.setValue(1)
            elif output_int == 8:
                self.radiobtn_out_20uAmV.click()
                self.slider_yamp.setValue(1)
            elif output_int == 7:
                self.radiobtn_out_10uAmV.click()
                self.slider_yamp.setValue(1)
            elif output_int == 6:
                self.radiobtn_out_5uAmV.click()
                self.slider_yamp.setValue(1)
            elif output_int == 5:
                self.radiobtn_out_2uAmV.click()
                self.slider_yamp.setValue(1)
            elif output_int == 4:
                self.radiobtn_out_1uAmV.click()
                self.slider_yamp.setValue(1)
            elif output_int == 3:
                self.radiobtn_out_500nAuV.click()
                self.slider_yamp.setValue(1)
            elif output_int == 2:
                self.radiobtn_out_200nAuV.click()
                self.slider_yamp.setValue(1)
            elif output_int == 1:
                self.radiobtn_out_100nAuV.click()
                self.slider_yamp.setValue(1)
            elif output_int == 0:
                self.radiobtn_out_50nAuV.click()
                self.slider_yamp.setValue(1)
            self.combobox_stepout.setCurrentIndex(output_int)

        def outputsel_func():
            self.dial_rsw_out.setValue(self.combobox_stepout.currentIndex())

        def output0_func():
            self.dial_rsw_out.setValue(0)

        def output1_func():
            self.dial_rsw_out.setValue(1)

        def output2_func():
            self.dial_rsw_out.setValue(2)

        def output3_func():
            self.dial_rsw_out.setValue(3)

        def output4_func():
            self.dial_rsw_out.setValue(4)

        def output5_func():
            self.dial_rsw_out.setValue(5)

        def output6_func():
            self.dial_rsw_out.setValue(6)

        def output7_func():
            self.dial_rsw_out.setValue(7)

        def output8_func():
            self.dial_rsw_out.setValue(8)

        def output9_func():
            self.dial_rsw_out.setValue(9)

        def output10_func():
            self.dial_rsw_out.setValue(10)

        def output11_func():
            self.dial_rsw_out.setValue(11)

        def output12_func():
            self.dial_rsw_out.setValue(12)

        def output13_func():
            self.dial_rsw_out.setValue(13)

        def output14_func():
            self.dial_rsw_out.setValue(14)

        def output15_func():
            self.dial_rsw_out.setValue(15)

        def settypedial_func():
            if self.dial_sw_type.value() == 1:
                self.slider_type.setValue(0)
            elif self.dial_sw_type.value() == 3:
                self.slider_type.setValue(1)
            elif self.dial_sw_type.value() == 2:
                self.dial_sw_type.setValue(3)
            else:
                self.dial_sw_type.setValue(1)
            self.combobox_dut_type.setCurrentIndex(self.slider_type.value())

        def settypesw_func():
            if self.slider_type.value() == 0:
                self.dial_sw_type.setValue(1)
            elif self.slider_type.value() == 1:
                self.dial_sw_type.setValue(3)

        def settypesel_func():
            self.slider_type.setValue(self.combobox_dut_type.currentIndex())

        def setpoldial_func():
            if self.dial_sw_pol.value() == 1:
                self.slider_pol.setValue(0)
            elif self.dial_sw_pol.value() == 3:
                self.slider_pol.setValue(1)
            elif self.dial_sw_pol.value() == 2:
                self.dial_sw_pol.setValue(3)
            else:
                self.dial_sw_pol.setValue(1)
            self.combobox_dut_pol.setCurrentIndex(self.slider_pol.value())

        def setpolsw_func():
            if self.slider_pol.value() == 0:
                self.dial_sw_pol.setValue(1)
            elif self.slider_pol.value() == 1:
                self.dial_sw_pol.setValue(3)

        def setpolsel_func():
            self.slider_pol.setValue(self.combobox_dut_pol.currentIndex())

        def setmethoddial_func():
            if self.dial_sw_method.value() == 2:
                self.slider_method.setValue(1)
            elif self.dial_sw_method.value() == 0:
                self.slider_method.setValue(0)
            elif self.dial_sw_method.value() == 4:
                self.slider_method.setValue(0)
            else:
                self.dial_sw_method.setValue(0)

        def setmethodsw_func():
            if self.slider_method.value() == 0:
                self.dial_sw_method.setValue(0)
            elif self.slider_method.value() == 1:
                self.dial_sw_method.setValue(2)

        def setyampdial_func():
            if self.dial_sw_yamp.value() == 1:
                self.slider_yamp.setValue(0)
                self.combobox_ch2_ratio.setCurrentIndex(1)
            elif self.dial_sw_yamp.value() == 3:
                self.slider_yamp.setValue(1)
                self.combobox_ch2_ratio.setCurrentIndex(0)
            elif self.dial_sw_yamp.value() == 2:
                self.dial_sw_yamp.setValue(3)
            else:
                self.dial_sw_yamp.setValue(1)

        def setyampsw_func():
            if self.slider_yamp.value() == 0:
                self.dial_sw_yamp.setValue(1)
            elif self.slider_yamp.value() == 1:
                self.dial_sw_yamp.setValue(3)

        def setdutdial_func():
            if self.dial_sw_dutsel.value() == 1:
                self.slider_dutsel.setValue(-1)
            elif self.dial_sw_dutsel.value() == 3:
                self.slider_dutsel.setValue(1)
            elif self.dial_sw_dutsel.value() == 0:
                self.slider_dutsel.setValue(0)
            elif self.dial_sw_dutsel.value() == 4:
                self.slider_dutsel.setValue(0)
            else:
                self.dial_sw_dutsel.setValue(0)

        def setdutsw_func():
            if self.slider_dutsel.value() == -1:
                self.dial_sw_dutsel.setValue(1)
            elif self.slider_dutsel.value() == 0:
                self.dial_sw_dutsel.setValue(0)
            elif self.slider_dutsel.value() == 1:
                self.dial_sw_dutsel.setValue(3)

        def setvrangedial_func():
            if self.dial_sw_vrange.value() == 1:
                self.slider_vrange.setValue(0)
                self.doublespinbox_vout.setMaximum(35)
                self.doublespinbox_vout.setSingleStep(.1)
                self.doublespinbox_vout.setDecimals(1)
                self.dial_pot_vrange.setMaximum(350)
                self.dial_pot_vrange.setSingleStep(1)
            elif self.dial_sw_vrange.value() == 3:
                self.slider_vrange.setValue(2)
                self.doublespinbox_vout.setMaximum(200)
                self.doublespinbox_vout.setSingleStep(1)
                self.doublespinbox_vout.setDecimals(0)
                self.dial_pot_vrange.setMaximum(2000)
                self.dial_pot_vrange.setSingleStep(10)
            elif self.dial_sw_vrange.value() == 0:
                self.slider_vrange.setValue(1)
                self.doublespinbox_vout.setMaximum(70)
                self.doublespinbox_vout.setSingleStep(1)
                self.doublespinbox_vout.setDecimals(0)
                self.dial_pot_vrange.setMaximum(700)
                self.dial_pot_vrange.setSingleStep(10)
            elif self.dial_sw_vrange.value() == 4:
                self.slider_vrange.setValue(1)
                self.doublespinbox_vout.setMaximum(70)
                self.doublespinbox_vout.setSingleStep(1)
                self.doublespinbox_vout.setDecimals(0)
                self.dial_pot_vrange.setMaximum(700)
                self.dial_pot_vrange.setSingleStep(10)
            else:
                self.dial_sw_vrange.setValue(1)
            self.combobox_vrange.setCurrentIndex(self.slider_vrange.value())

        def setvrangesw_func():
            global irange_int
            irange_int = int((self.dial_rsw_irange.value() - 5) / 2)
            global irange_arr_val
            if self.slider_vrange.value() == 0:
                iout = 2000
            elif self.slider_vrange.value() == 1:
                iout = 1000
            else:
                iout = 100
            self.dial_pot_irange.setValue(int(self.spinbox_iout.value() / iout / irange_arr_val[irange_int] * 100))
            if self.slider_vrange.value() == 0:
                self.dial_sw_vrange.setValue(1)
            elif self.slider_vrange.value() == 1:
                self.dial_sw_vrange.setValue(0)
            elif self.slider_vrange.value() == 2:
                self.dial_sw_vrange.setValue(3)

        def setvrangesel_func():
            self.slider_vrange.setValue(self.combobox_vrange.currentIndex())

        def setorangedial_func():
            if self.dial_sw_offsetdir.value() == 0:
                self.slider_offsetdir.setValue(0)
            elif self.dial_sw_offsetdir.value() == 4:
                self.slider_offsetdir.setValue(0)
            elif self.dial_sw_offsetdir.value() == 2:
                self.slider_offsetdir.setValue(2)
            elif self.dial_sw_offsetdir.value() == 3:
                self.slider_offsetdir.setValue(1)
            else:
                self.dial_sw_offsetdir.setValue(3)

        def setorangesw_func():
            if self.slider_offsetdir.value() == 0:
                self.dial_sw_offsetdir.setValue(0)
                self.label_voffset.setText("V-Offset [ (-) V]")
                self.dial_pot_offset.setValue(self.doublespinbox_voffset.value() * 10)
            elif self.slider_offsetdir.value() == 1:
                self.dial_sw_offsetdir.setValue(3)
                self.label_voffset.setText("V-Offset [V]")
                self.dial_pot_offset.setValue(0)
            elif self.slider_offsetdir.value() == 2:
                self.dial_sw_offsetdir.setValue(2)
                self.label_voffset.setText("V-Offset [ (+) V]")
                self.dial_pot_offset.setValue(self.doublespinbox_voffset.value() * 10)

        def setoffset_func():
            self.doublespinbox_voffset.setValue(self.dial_pot_offset.value() / 10)

        def setoffsetsel_func():
            if not self.slider_offsetdir.value() == 1:
                self.dial_pot_offset.setValue(self.doublespinbox_voffset.value() * 10)

        def setvout_func():
            self.doublespinbox_vout.setValue(self.dial_pot_vrange.value() / 10)

        def setvoutsel_func():
            self.dial_pot_vrange.setValue(self.doublespinbox_vout.value() * 10)

        def setstepdly_func():
            self.spinbox_stepdly.setValue(self.dial_pot_stepdly.value())

        def setstepdlysel_func():
            self.dial_pot_stepdly.setValue(self.spinbox_stepdly.value())

        def setirange_func():
            global irange_int
            irange_int = int((self.dial_rsw_irange.value() - 5) / 2)
            global irange_arr_val
            if self.slider_vrange.value() == 0:
                iout = 2000
            elif self.slider_vrange.value() == 1:
                iout = 1000
            else:
                iout = 100
            self.dial_pot_irange.setValue(int(self.spinbox_iout.value() / iout / irange_arr_val[irange_int] * 100))

        self.spinbox_iout.valueChanged.connect(setirange_func)
        self.spinbox_stepdly.valueChanged.connect(setstepdlysel_func)
        self.dial_pot_stepdly.valueChanged.connect(setstepdly_func)
        self.doublespinbox_vout.valueChanged.connect(setvoutsel_func)
        self.dial_pot_vrange.valueChanged.connect(setvout_func)
        self.slider_offsetdir.valueChanged.connect(setorangesw_func)
        self.dial_sw_offsetdir.valueChanged.connect(setorangedial_func)
        self.dial_pot_offset.valueChanged.connect(setoffset_func)
        self.doublespinbox_voffset.valueChanged.connect(setoffsetsel_func)
        self.slider_vrange.valueChanged.connect(setvrangesw_func)
        self.dial_sw_vrange.valueChanged.connect(setvrangedial_func)
        self.combobox_vrange.currentIndexChanged.connect(setvrangesel_func)
        self.slider_dutsel.valueChanged.connect(setdutsw_func)
        self.dial_sw_dutsel.valueChanged.connect(setdutdial_func)
        self.slider_yamp.valueChanged.connect(setyampsw_func)
        self.dial_sw_yamp.valueChanged.connect(setyampdial_func)
        self.slider_method.valueChanged.connect(setmethodsw_func)
        self.dial_sw_method.valueChanged.connect(setmethoddial_func)
        self.combobox_dut_pol.currentIndexChanged.connect(setpolsel_func)
        self.combobox_dut_type.currentIndexChanged.connect(settypesel_func)
        self.slider_pol.valueChanged.connect(setpolsw_func)
        self.dial_sw_pol.valueChanged.connect(setpoldial_func)
        self.slider_type.valueChanged.connect(settypesw_func)
        self.dial_sw_type.valueChanged.connect(settypedial_func)
        self.dial_rsw_steps.valueChanged.connect(step_func)
        self.radiobtn_steps_0.clicked.connect(step0_func)
        self.radiobtn_steps_1.clicked.connect(step1_func)
        self.radiobtn_steps_2.clicked.connect(step2_func)
        self.radiobtn_steps_3.clicked.connect(step3_func)
        self.radiobtn_steps_4.clicked.connect(step4_func)
        self.radiobtn_steps_5.clicked.connect(step5_func)
        self.radiobtn_steps_6.clicked.connect(step6_func)
        self.radiobtn_steps_7.clicked.connect(step7_func)
        self.spinbox_steps.valueChanged.connect(stepsel_func)
        self.dial_rsw_irange.valueChanged.connect(irange_func)
        self.radiobtn_irange_x1.clicked.connect(irange0_func)
        self.radiobtn_irange_x05.clicked.connect(irange1_func)
        self.radiobtn_irange_x02.clicked.connect(irange2_func)
        self.radiobtn_irange_x01.clicked.connect(irange3_func)
        self.radiobtn_irange_x005.clicked.connect(irange4_func)
        self.radiobtn_irange_x002.clicked.connect(irange5_func)
        self.combobox_irange.currentIndexChanged.connect(irangesel_func)
        self.dial_rsw_out.valueChanged.connect(output_func)
        self.radiobtn_out_50nAuV.clicked.connect(output0_func)
        self.radiobtn_out_100nAuV.clicked.connect(output1_func)
        self.radiobtn_out_200nAuV.clicked.connect(output2_func)
        self.radiobtn_out_500nAuV.clicked.connect(output3_func)
        self.radiobtn_out_1uAmV.clicked.connect(output4_func)
        self.radiobtn_out_2uAmV.clicked.connect(output5_func)
        self.radiobtn_out_5uAmV.clicked.connect(output6_func)
        self.radiobtn_out_10uAmV.clicked.connect(output7_func)
        self.radiobtn_out_20uAmV.clicked.connect(output8_func)
        self.radiobtn_out_50uAmV.clicked.connect(output9_func)
        self.radiobtn_out_100uAmV.clicked.connect(output10_func)
        self.radiobtn_out_200uAmV.clicked.connect(output11_func)
        self.radiobtn_out_500uAmV.clicked.connect(output12_func)
        self.radiobtn_out_1mAV.clicked.connect(output13_func)
        self.radiobtn_out_2mAV.clicked.connect(output14_func)
        self.radiobtn_out_5mAV.clicked.connect(output15_func)
        self.combobox_stepout.currentIndexChanged.connect(outputsel_func)

        # print(irange_arr[irange_int])


if __name__ == "__main__":
    app = QApplication(sys.argv)
    win = Window()
    win.show()
    sys.exit(app.exec())
